@@ -19,21 +19,19 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
 #include "pango-tabs.h"
-#include "pango-impl-utils.h"
 #include <string.h>
 
 typedef struct _PangoTab PangoTab;
 
 struct _PangoTab
 {
-  gint location;	        /* Offset in pixels of this tab stop
-				 * from the left margin of the text.
-				 */
+  gint location; 	        /* Offset in pixels of this tab stop
+                                 * from the left margin of the text.
+                                 */
   PangoTabAlign alignment;      /* Where the tab stop appears relative
-				 * to the text.
-				 */
+                                 * to the text.
+                                 */
 };
 
 struct _PangoTabArray
@@ -59,17 +57,17 @@ init_tabs (PangoTabArray *array, gint start, gint end)
  * pango_tab_array_new:
  * @initial_size: Initial number of tab stops to allocate, can be 0
  * @positions_in_pixels: whether positions are in pixel units
- *
+ * 
  * Creates an array of @initial_size tab stops. Tab stops are specified in
  * pixel units if @positions_in_pixels is %TRUE, otherwise in Pango
  * units. All stops are initially at position 0.
- *
+ * 
  * Return value: the newly allocated #PangoTabArray, which should
  *               be freed with pango_tab_array_free().
  **/
 PangoTabArray*
 pango_tab_array_new (gint initial_size,
-		     gboolean positions_in_pixels)
+                     gboolean positions_in_pixels)
 {
   PangoTabArray *array;
 
@@ -80,7 +78,7 @@ pango_tab_array_new (gint initial_size,
    * If we allowed tab array resizing we'd need to drop this
    * optimization.
    */
-  array = g_slice_new (PangoTabArray);
+  array = g_new (PangoTabArray, 1);
   array->size = initial_size;
   array->allocated = initial_size;
 
@@ -93,7 +91,7 @@ pango_tab_array_new (gint initial_size,
     array->tabs = NULL;
 
   array->positions_in_pixels = positions_in_pixels;
-
+  
   return array;
 }
 
@@ -104,28 +102,28 @@ pango_tab_array_new (gint initial_size,
  * @first_alignment: alignment of first tab stop
  * @first_position: position of first tab stop
  * @varargs: additional alignment/position pairs
- *
+ * 
  * This is a convenience function that creates a #PangoTabArray
  * and allows you to specify the alignment and position of each
  * tab stop. You <emphasis>must</emphasis> provide an alignment
  * and position for @size tab stops.
- *
+ * 
  * Return value: the newly allocated #PangoTabArray, which should
  *               be freed with pango_tab_array_free().
  **/
 PangoTabArray  *
 pango_tab_array_new_with_positions (gint           size,
-				    gboolean       positions_in_pixels,
-				    PangoTabAlign  first_alignment,
-				    gint           first_position,
-				    ...)
+                                    gboolean       positions_in_pixels,
+                                    PangoTabAlign  first_alignment,
+                                    gint           first_position,
+                                    ...)
 {
   PangoTabArray *array;
   va_list args;
   int i;
 
   g_return_val_if_fail (size >= 0, NULL);
-
+  
   array = pango_tab_array_new (size, positions_in_pixels);
 
   if (size == 0)
@@ -136,7 +134,7 @@ pango_tab_array_new_with_positions (gint           size,
 
   if (size == 1)
     return array;
-
+  
   va_start (args, first_position);
 
   i = 1;
@@ -160,20 +158,20 @@ GType
 pango_tab_array_get_type (void)
 {
   static GType our_type = 0;
-
-  if (G_UNLIKELY (our_type == 0))
-    our_type = g_boxed_type_register_static (I_("PangoTabArray"),
-					     (GBoxedCopyFunc)pango_tab_array_copy,
-					     (GBoxedFreeFunc)pango_tab_array_free);
+  
+  if (our_type == 0)
+    our_type = g_boxed_type_register_static ("PangoTabArray",
+                                             (GBoxedCopyFunc)pango_tab_array_copy,
+                                             (GBoxedFreeFunc)pango_tab_array_free);
   return our_type;
 }
 
 /**
  * pango_tab_array_copy:
  * @src: #PangoTabArray to copy
- *
+ * 
  * Copies a #PangoTabArray
- *
+ * 
  * Return value: the newly allocated #PangoTabArray, which should
  *               be freed with pango_tab_array_free().
  **/
@@ -183,7 +181,7 @@ pango_tab_array_copy (PangoTabArray *src)
   PangoTabArray *copy;
 
   g_return_val_if_fail (src != NULL, NULL);
-
+  
   copy = pango_tab_array_new (src->size, src->positions_in_pixels);
 
   memcpy (copy->tabs, src->tabs, sizeof(PangoTab)*src->size);
@@ -196,7 +194,7 @@ pango_tab_array_copy (PangoTabArray *src)
  * @tab_array: a #PangoTabArray
  *
  * Frees a tab array and associated resources.
- *
+ * 
  **/
 void
 pango_tab_array_free   (PangoTabArray *tab_array)
@@ -204,23 +202,23 @@ pango_tab_array_free   (PangoTabArray *tab_array)
   g_return_if_fail (tab_array != NULL);
 
   g_free (tab_array->tabs);
-
-  g_slice_free (PangoTabArray, tab_array);
+  
+  g_free (tab_array);
 }
 
 /**
  * pango_tab_array_get_size:
  * @tab_array: a #PangoTabArray
- *
+ * 
  * Gets the number of tab stops in @tab_array.
- *
+ * 
  * Return value: the number of tab stops in the array.
  **/
 gint
 pango_tab_array_get_size (PangoTabArray *tab_array)
 {
   g_return_val_if_fail (tab_array != NULL, 0);
-
+  
   return tab_array->size;
 }
 
@@ -228,32 +226,32 @@ pango_tab_array_get_size (PangoTabArray *tab_array)
  * pango_tab_array_resize:
  * @tab_array: a #PangoTabArray
  * @new_size: new size of the array
- *
+ * 
  * Resizes a tab array. You must subsequently initialize any tabs that
  * were added as a result of growing the array.
  *
  **/
 void
 pango_tab_array_resize (PangoTabArray *tab_array,
-			gint           new_size)
+                        gint           new_size)
 {
   if (new_size > tab_array->allocated)
     {
       gint current_end = tab_array->allocated;
-
+      
       /* Ratchet allocated size up above the index. */
       if (tab_array->allocated == 0)
-	tab_array->allocated = 2;
-
+        tab_array->allocated = 2;
+      
       while (new_size > tab_array->allocated)
-	tab_array->allocated = tab_array->allocated * 2;
-
+        tab_array->allocated = tab_array->allocated * 2;
+      
       tab_array->tabs = g_renew (PangoTab, tab_array->tabs,
 				 tab_array->allocated);
-
+      
       init_tabs (tab_array, current_end, tab_array->allocated);
     }
-
+  
   tab_array->size = new_size;
 }
 
@@ -267,13 +265,13 @@ pango_tab_array_resize (PangoTabArray *tab_array,
  * Sets the alignment and location of a tab stop.
  * @alignment must always be #PANGO_TAB_LEFT in the current
  * implementation.
- *
+ * 
  **/
 void
 pango_tab_array_set_tab  (PangoTabArray *tab_array,
-			  gint           tab_index,
-			  PangoTabAlign  alignment,
-			  gint           location)
+                          gint           tab_index,
+                          PangoTabAlign  alignment,
+                          gint           location)
 {
   g_return_if_fail (tab_array != NULL);
   g_return_if_fail (tab_index >= 0);
@@ -282,7 +280,7 @@ pango_tab_array_set_tab  (PangoTabArray *tab_array,
 
   if (tab_index >= tab_array->size)
     pango_tab_array_resize (tab_array, tab_index + 1);
-
+  
   tab_array->tabs[tab_index].alignment = alignment;
   tab_array->tabs[tab_index].location = location;
 }
@@ -295,13 +293,13 @@ pango_tab_array_set_tab  (PangoTabArray *tab_array,
  * @location: location to store tab position, or %NULL
  *
  * Gets the alignment and position of a tab stop.
- *
+ * 
  **/
 void
 pango_tab_array_get_tab  (PangoTabArray *tab_array,
-			  gint           tab_index,
-			  PangoTabAlign *alignment,
-			  gint          *location)
+                          gint           tab_index,
+                          PangoTabAlign *alignment,
+                          gint          *location)
 {
   g_return_if_fail (tab_array != NULL);
   g_return_if_fail (tab_index < tab_array->size);
@@ -323,15 +321,15 @@ pango_tab_array_get_tab  (PangoTabArray *tab_array,
  * If non-%NULL, @alignments and @locations are filled with allocated
  * arrays of length pango_tab_array_get_size(). You must free the
  * returned array.
- *
+ * 
  **/
 void
 pango_tab_array_get_tabs (PangoTabArray *tab_array,
-			  PangoTabAlign **alignments,
-			  gint          **locations)
+                          PangoTabAlign **alignments,
+                          gint          **locations)
 {
   gint i;
-
+  
   g_return_if_fail (tab_array != NULL);
 
   if (alignments)
@@ -339,14 +337,14 @@ pango_tab_array_get_tabs (PangoTabArray *tab_array,
 
   if (locations)
     *locations = g_new (gint, tab_array->size);
-
+  
   i = 0;
   while (i < tab_array->size)
     {
       if (alignments)
-	(*alignments)[i] = tab_array->tabs[i].alignment;
+        (*alignments)[i] = tab_array->tabs[i].alignment;
       if (locations)
-	(*locations)[i] = tab_array->tabs[i].location;
+        (*locations)[i] = tab_array->tabs[i].location;
 
       ++i;
     }
@@ -355,16 +353,16 @@ pango_tab_array_get_tabs (PangoTabArray *tab_array,
 /**
  * pango_tab_array_get_positions_in_pixels:
  * @tab_array: a #PangoTabArray
- *
+ * 
  * Returns %TRUE if the tab positions are in pixels, %FALSE if they are
  * in Pango units.
- *
+ * 
  * Return value: whether positions are in pixels.
  **/
 gboolean
 pango_tab_array_get_positions_in_pixels (PangoTabArray *tab_array)
 {
   g_return_val_if_fail (tab_array != NULL, FALSE);
-
+  
   return tab_array->positions_in_pixels;
 }
